import tomllib
from pathlib import Path
from typing import Optional

import typer

from savit.config import CONFIG_FILE, set_env

app = typer.Typer(help="Helping you to write docs by saving your commands")


@app.command()
def config(
    open_file: bool = typer.Option(False, "--open-file", help="Opens the config file")
):
    """
    Saves your configurations to ~/.config/savit/config.toml
    """
    if open_file:
        typer.launch(str(CONFIG_FILE), locate=True)
        raise typer.Exit()
    if CONFIG_FILE.exists():
        print("Config file already exists")
        raise typer.Exit()

    set_env()


@app.command()
def start():
    """
    Start saving your commands
    """

    if not CONFIG_FILE.exists():
        set_env()

    saving = typer.style("saving", fg=typer.colors.GREEN)
    savit_stop = typer.style("savit stop", fg=typer.colors.BLUE)
    typer.echo(
        f"""
               Started {saving}!
               Use your commands and when you are done use {savit_stop} 
               Don't erase your shell history file!
               """
    )


@app.command()
def stop(
    txt: bool = typer.Option(False, "--txt", help="Saves your commands to a .txt file"),
    md: bool = typer.Option(False, "--md", help="Saves your commands to a .md file"),
    file: Optional[Path] = typer.Option(
        None, help="File (may include path) to save your commands"
    ),
):
    """
    Stop saving your commands and writes them to a file
    """
    try:
        with CONFIG_FILE.open("rb") as f:
            config_toml = tomllib.load(f)
            hist_path = config_toml["savit"]["history_path"]

        with open(hist_path, "r") as hist:
            hist_list = hist.readlines()
            start_index = len(hist_list) - 1 - hist_list[::-1].index("savit start\n")
            saved_commands = hist_list[start_index + 1 : -1]

        output_format = config_toml["savit"]["output_format"]
        if file is None:
            if txt:
                output_format = "txt"
            elif md:
                output_format = "md"
            else:
                output_format = config_toml["savit"]["output_format"]

            output_folder = config_toml["savit"]["output_folder"]
            output_file = str(Path(output_folder) / f"commands.{output_format}")
        elif txt or md:
            typer.secho(
                "You can't use --txt or --md with --file. Use only one of them",
                fg=typer.colors.RED,
                err=True,
            )
            raise typer.Exit()

        elif not file.is_absolute():
            output_folder = config_toml["savit"]["output_folder"]
            output_file = str(Path(output_folder) / file)
            print("não é absoluto")

        else:
            output_file = str(file)

        with open(output_file, "w") as commands:
            for command in saved_commands:
                if output_format == "txt" or output_file.endswith(".txt"):
                    commands.write(f"{command}")
                elif output_format == "md" or output_file.endswith(".md"):
                    commands.write(f"```console\n{command}```\n\n")
            print("Commands saved")
    except FileNotFoundError as e:
        typer.echo(f"File not found: {e}", err=True)
    except Exception as e:
        typer.echo(f"Error: {e}", err=True)


@app.callback()
def main():
    """
    Save your commands
    """
    ...

if __name__ == "__main__":
    app()



import os
import tomllib
from pathlib import Path

import typer
from click import Choice

HISTORY_PATH = ""
DOT_CONFIG_FILE = Path(typer.get_app_dir("savit"))
CONFIG_FILE: Path = Path(DOT_CONFIG_FILE) / "config.toml"


def set_env():
    DOT_CONFIG_FILE.mkdir(parents=True, exist_ok=True)
    CONFIG_FILE.touch(exist_ok=True)

    shell = os.getenv("SHELL")
    home = Path.home()
    if shell.find("zsh") != -1:
        config_toml_string = f"""[savit]
history_path = "{home}/.zhistory"
"""
        with CONFIG_FILE.open("w") as f:
            f.write(config_toml_string)

        with CONFIG_FILE.open("rb") as f:
            config_toml = tomllib.load(f)
        HISTORY_PATH = config_toml["savit"]["history_path"]

    elif shell.find("bash") != -1:
        config_toml_string = f"""[savit]
history_path = "{home}/.bash_history"
"""
        with CONFIG_FILE.open("w") as f:
            f.write(config_toml_string)

        with CONFIG_FILE.open("rb") as f:
            config_toml = tomllib.load(f)

        HISTORY_PATH = config_toml["savit"]["history_path"]

    elif shell.find("fish") != -1:
        config_toml_string = f"""[savit]
history_path = "{home}/.local/share/fish/fish_history"
"""
        with CONFIG_FILE.open("w") as f:
            f.write(config_toml_string)

        with CONFIG_FILE.open("rb") as f:
            config_toml = tomllib.load(f)

        HISTORY_PATH = config_toml["savit"]["history_path"]

    print(HISTORY_PATH)

    if confirmation_prompt := typer.confirm(
        "is this your history file?", default=True
    ):
        print("Config file saved")

    else:
        HISTORY_PATH = Path(input("Enter the path of your history file: "))
        config_toml_string = f"""[savit]
history_path = {HISTORY_PATH}"""
        with CONFIG_FILE.open("w") as f:
            f.write(config_toml_string)

        print(f"{HISTORY_PATH} saved")

    output_format = typer.prompt(
        "Choose an output format", type=Choice(["txt", "md"]), show_choices=True
    )
    output_folder = input(
        "Type the path to the folder where commands should be store (leave blank to save on current usage directory): "
    )

    with CONFIG_FILE.open("a") as f:
        f.write(f'output_format = "{output_format}"\n')
        if output_folder:
            f.write(f'output_folder = "{output_folder}"\n')
        else:
            f.write('output_folder = "./"')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['savit', 'savit.config']

package_data = \
{'': ['*']}

install_requires = \
['typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['savit = savit.main:app']}

setup_kwargs = {
    'name': 'savit',
    'version': '0.1.1',
    'description': 'Saves your commands as you use them',
    'long_description': '# `savit`\n\nHelping you to write docs by saving your commands\n\n**Usage**:\n\n```console\n$ savit [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--install-completion`: Install completion for the current shell.\n* `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `config`: Saves your configurations to ~/.config/savit/config.toml\n* `start`: Start saving your commands\n* `stop`: Stop saving your commands and writes them to a file\n\n## `savit config`\n\nSaves your configurations to ~/.config/savit/config.toml\n\n**Usage**:\n\n```console\n$ savit config [OPTIONS]\n```\n\n**Options**:\n\n* `--open-file`: Opens the config file  [default: False]\n* `--help`: Show this message and exit.\n\n## `savit start`\n\nStart saving your commands\n\n**Usage**:\n\n```console\n$ savit start [OPTIONS]\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `savit stop`\n\nStop saving your commands and writes them to a file\n\n**Usage**:\n\n```console\n$ savit stop [OPTIONS]\n```\n\n**Options**:\n\n* `--txt`: Saves your commands to a .txt file  [default: False]\n* `--md`: Saves your commands to a .md file  [default: False]\n* `--file PATH`: File (may include path) to save your commands\n* `--help`: Show this message and exit.\n\n\n**Points of attention**\n* Don\'t use aliases to savit commands, since savit works by reading your shell history, the app will work in an unexpected way if aliases are used;\n* Make sure to have your config file at `~/.config/savit/config.toml` correctly set up;\n\n\n**Config file structure**\n```toml\n[savit]\nhistory_path = "" # path to your shell history file\noutput_format = "" # output format to use by deafault (txt or md)\noutput_folder = "" # folder to save your commands by default (use ./ to save commands from the location where savit runs)\n```',
    'author': 'ivansantiagojr',
    'author_email': 'ivansantiago.junior@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
